/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: xline.cxx,v $
 * $Revision: 1.20 $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_svx.hxx"

// #include <math.h>

#include <tools/bigint.hxx>
#include <tools/poly.hxx>
#include <vcl/svapp.hxx>
#include <vcl/virdev.hxx>
#include <vcl/metaact.hxx>
#include <vcl/gdimtf.hxx>
#include <vcl/gradient.hxx>
#include <svx/xpoly.hxx>
#include <svx/xoutx.hxx>
#include <svtools/colorcfg.hxx>

//#ifndef _BGFX_POLYGON_B2DPOLYGON_HXX
//#include <basegfx/polygon/b2dpolygon.hxx>
//#endif

#ifndef _BGFX_POLYGON_B2DPOLYPOLYGONTOOLS_HXX
#include <basegfx/polygon/b2dpolypolygontools.hxx>
#endif
#include <basegfx/matrix/b2dhommatrix.hxx>

#include <algorithm>

#define GLOBALOVERFLOW

/*************************************************************************
|*
|*    Parameterklassse fuer dicke Linien mit Linienmuster
|*
\************************************************************************/

class XLineParam
{
 public:
	long    nPatSeg;
	long    nPatRemain;
	BOOL    bHasJoin, bUseJoin3;
	Point   aJoin1, aJoin2, aJoin3;
	double  fLength;
	long    nLineDx, nLineDy;
	long    nDxW, nDyW;

	XLineParam() {}

	void Init(const Point& rP1, const Point& rP2, long nWidth);
};

void XLineParam::Init(const Point& rP1, const Point& rP2, long nWidth)
{
	double fWidth = 0, fRound;

	nPatSeg = -1;
	nPatRemain = 0;
	bHasJoin = FALSE;
	bUseJoin3 = FALSE;
	nLineDx = rP2.X() - rP1.X();
	nLineDy = rP2.Y() - rP1.Y();
	fLength = sqrt((double) nLineDx * nLineDx + (double) nLineDy * nLineDy);
	if ( fLength > 0 )
		fWidth = nWidth / fLength;
	if ( nLineDy >= 0 ) fRound =  0.5;
	else                fRound = -0.5;
	nDxW =   (long) (fWidth * nLineDy + fRound);
	if ( nLineDx >= 0 ) fRound =  0.5;
	else                fRound = -0.5;
	nDyW = - (long) (fWidth * nLineDx + fRound);

	aJoin1.X() = rP2.X() + nDxW / 2;
	aJoin1.Y() = rP2.Y() + nDyW / 2;
	aJoin2 = aJoin1;
	aJoin2.X() -= nDxW;
	aJoin2.Y() -= nDyW;
	aJoin3 = aJoin1;
}

/*************************************************************************
|*
|*    XOutputDevice::CalcFatLineJoin()
|*
|*    Beschreibung      Uebergang zwischen zwei Linien eines Polygons
|*                      berechnen
|*    Ersterstellung    02.12.94
|*    Letzte Aenderung  09.01.95
|*
\************************************************************************/

void XOutputDevice::CalcFatLineJoin(const Point& rEnd, const Point& rNext,
									XLineParam& rParam)
{
	long nNextDx = rNext.X() - rEnd.X();
	long nNextDy = rNext.Y() - rEnd.Y();

	if ( nNextDx || nNextDy )
	{
		double fRound;
		long nJoinDx, nJoinDy;
		BOOL bFlatJoin = FALSE;
		BOOL bResetJoin = FALSE;

		double fNextLen = sqrt((double) nNextDx * nNextDx +
							   (double) nNextDy * nNextDy);

		// Berechnung des Anschlussuebergangs durch Auswertung
		// der Seitenverhaeltnisse in den Uebergangsdreiecken
		long nPrevDxW = rParam.nDxW;
		long nPrevDyW = rParam.nDyW;

		double fWidth = (double) nLineWidth / fNextLen;
		if ( nNextDy >= 0 ) fRound =  0.5;
		else                fRound = -0.5;
		long nNextDxW =   (long) (fWidth * nNextDy + fRound);
		if ( nNextDx >= 0 ) fRound =  0.5;
		else                fRound = -0.5;
		long nNextDyW = - (long) (fWidth * nNextDx + fRound);
		long nDxA = nNextDxW - nPrevDxW;
		long nDyA = nNextDyW - nPrevDyW;
		long nDxU = nPrevDxW + nNextDxW;
		long nDyU = nPrevDyW + nNextDyW;
		double fJoin = nDxA * nDxA + nDyA * nDyA;
		double fULen = nDxU * nDxU + nDyU * nDyU;

		if ( fULen > 0 )    fJoin = sqrt(fJoin / fULen) / 2;
		else                fJoin = 0;

		if ( fJoin > 0.7 )
		{
			double fJoinLen = fJoin * nLineWidth;
			double fLen = rParam.fLength < fNextLen ? rParam.fLength : fNextLen;

			if ( fJoinLen > fLen )
			{
				fJoin = 0;
				if ( fLen == rParam.fLength )
					bResetJoin = TRUE;
			}
			bFlatJoin = TRUE;
		}

		if ( nPrevDyW >= 0 )    fRound =  0.5;
		else                    fRound = -0.5;
		nJoinDx = -(long) (fJoin * nPrevDyW + fRound);

		if ( nPrevDxW >= 0 )    fRound =  0.5;
		else                    fRound = -0.5;
		nJoinDy = (long) (fJoin * nPrevDxW + fRound);

		// mit Vektorprodukt feststellen, ob Anschlusslinie links oder rechts
		// von der ersten Linie verlaeuft; < 0 -> links
		BOOL bLeftTurn =( (rParam.nLineDx * nNextDy -
						   rParam.nLineDy * nNextDx) < 0 );

		if ( bLeftTurn )
		{
			nJoinDx = - nJoinDx;
			nJoinDy = - nJoinDy;
		}
		rParam.bUseJoin3 = FALSE;

		if ( bFlatJoin )
		{
			if ( bLeftTurn )
			{
				rParam.aJoin3 = rParam.aJoin1;
				rParam.aJoin1 = rParam.aJoin2;
				if ( bResetJoin )
					rParam.aJoin3 = rParam.aJoin2;
				else
				{
					rParam.aJoin3.X() += nJoinDx;
					rParam.aJoin3.Y() += nJoinDy;
				}
				rParam.aJoin2.X() = rEnd.X() - nNextDxW / 2;
				rParam.aJoin2.Y() = rEnd.Y() - nNextDyW / 2;
				rParam.bUseJoin3 = TRUE;
			}
			else
			{
				if ( bResetJoin )
					rParam.aJoin2 = rParam.aJoin1;
				else
				{
					rParam.aJoin2.X() -= nJoinDx;
					rParam.aJoin2.Y() -= nJoinDy;
				}
				rParam.aJoin3.X() = rEnd.X() + nNextDxW / 2;
				rParam.aJoin3.Y() = rEnd.Y() + nNextDyW / 2;
			}
		}
		else
		{
			rParam.aJoin1.X() += nJoinDx;
			rParam.aJoin1.Y() += nJoinDy;
			rParam.aJoin2.X() -= nJoinDx;
			rParam.aJoin2.Y() -= nJoinDy;
			rParam.aJoin3 = rParam.aJoin1;
		}
		rParam.bHasJoin = TRUE;

		rParam.fLength = fNextLen;
		rParam.nLineDx = nNextDx;
		rParam.nLineDy = nNextDy;
		rParam.nDxW = nNextDxW;
		rParam.nDyW = nNextDyW;
	}
}

/*************************************************************************
|*
|*    XOutputDevice::DrawStartEndPoly()
|*
|*    Linienanfang- bzw. -endpolygon zeichnen
|*    Ersterstellung    17.01.95 ESO
|*    Letzte Aenderung  18.01.95 ESO
|*
\************************************************************************/

void XOutputDevice::DrawStartEndPoly(const Point& rPoint, const basegfx::B2DPolyPolygon& rPolyPolygon, const XLineParam& rParam)
{
	basegfx::B2DPolyPolygon aPolyPolygon(rPolyPolygon);
	basegfx::B2DHomMatrix aMatrix;

	// add rotation
	if(0.0 != rParam.fLength)
	{
		double fSin((double) rParam.nLineDx / -rParam.fLength);
		double fCos((double) rParam.nLineDy / -rParam.fLength);
		
		aMatrix.set(0, 0, fCos);
		aMatrix.set(1, 1, fCos);
		aMatrix.set(1, 0, fSin);
		aMatrix.set(0, 1, -fSin);
	}

	// add translation
	aMatrix.translate(rPoint.getX(), rPoint.getY());

	// apply to polygon
	aPolyPolygon.transform(aMatrix);

	// create Polygon from subdivided B2DPolygon and paint
	const PolyPolygon aDrawPolyPolygon(basegfx::tools::adaptiveSubdivideByAngle(aPolyPolygon));
	pOut->DrawPolyPolygon(aDrawPolyPolygon);
}

/*************************************************************************
|*
|*    XOutputDevice::DrawLineStartEnd()
|*
|*    Linienanfang bzw. -ende eines Polygons zeichnen
|*    Ersterstellung    18.01.95 ESO
|*    Letzte Aenderung  20.01.95 ESO
|*
\************************************************************************/

void XOutputDevice::DrawLineStartEnd(const Polygon& rPoly)
{
	Point   aDiff;
	USHORT  nPntMax = rPoly.GetSize() - 1;
	USHORT  i = 0;

	if ( bLineStart || bLineEnd )
	{   // Linien mit Laenge 0 nicht beruecksichtigen
		while ( i < nPntMax )
		{
			aDiff = rPoly[i+1] - rPoly[i];
			if ( aDiff.X() || aDiff.Y() )
				break;
			i++;
		}
		while ( nPntMax > i )
		{
			aDiff = rPoly[nPntMax] - rPoly[0];
			if ( aDiff.X() || aDiff.Y() )
				break;
			nPntMax--;
		}
		if ( i < nPntMax )
		{
			XLineParam  aLineParam;
			const Color	aLineColor2( pOut->GetLineColor() );
			const Color	aFillColor2( pOut->GetFillColor() );

			pOut->SetLineColor();
			pOut->SetFillColor( aLineColor2 );

			if( bLineStart )
			{
				aLineParam.Init(rPoly[i], rPoly[i+1], 1);
				DrawStartEndPoly(rPoly[i], maLineStartPolyPolygon, aLineParam);
			}
			if ( bLineEnd )
			{
				aLineParam.Init(rPoly[nPntMax], rPoly[nPntMax-1], 1);
				DrawStartEndPoly(rPoly[nPntMax], maLineEndPolyPolygon, aLineParam);
			}
			
			pOut->SetFillColor( aFillColor2 );
			pOut->SetLineColor( aLineColor2 );
		}
	}
}

/*************************************************************************
|*
|*    XOutputDevice::DrawFatLine()
|*
|*    Beschreibung      Dicke Linie mit oder ohne Linienstile zeichnen
|*    Ersterstellung    28.11.94
|*    Letzte Aenderung  09.10.95
|*
\************************************************************************/

void XOutputDevice::DrawFatLine(const Point& rStart, const Point& rEnd,
							const Point* pNext, XLineParam& rParam)
{
	Polygon aPoly(5);
	BOOL    bLineComplete = FALSE;

	long nLineDx = rParam.nLineDx;
	long nLineDy = rParam.nLineDy;
	double fLength = rParam.fLength;

	long nDxW = rParam.nDxW;
	long nDyW = rParam.nDyW;

	double  fDx = 0, fDy = 0;
	long    nDx, nDy;
	long    nSeg = rParam.nPatSeg;
	long    nPattern;

	// Linienmuster vorhanden?
	if ( !pLinePattern )
	{
		nPattern = -1;
		nSeg = 0;
	}
	else
	{
		// Angefangenes Segment zu Ende zeichnen?
		if ( rParam.nPatRemain )
			nPattern = rParam.nPatRemain;
		else    // sonst naechstes Segment
		{
			nSeg++;
			if ( pLinePattern[nSeg] == 0 )  nSeg = 0;
			nPattern = pLinePattern[nSeg];
		}
	}
	aPoly[0].X() = rStart.X() + nDxW / 2;
	aPoly[0].Y() = rStart.Y() + nDyW / 2;
	aPoly[1].X() = aPoly[0].X() - nDxW;
	aPoly[1].Y() = aPoly[0].Y() - nDyW;
	aPoly[2] = aPoly[1];
	aPoly[3] = aPoly[0];
	aPoly[4] = aPoly[0];
	// An vorheriges Segment anschliessen?
	if ( rParam.bHasJoin )
	{
		aPoly[0] = rParam.aJoin1;
		aPoly[1] = rParam.aJoin2;
		aPoly[4] = rParam.aJoin3;
	}
	// Punkte zum Testen auf erreichen des Linienendes
	Point SegStart = rStart;
	Point SegEnd   = rStart;

	// Anschlusspunkte zunaechst im rechten Winkel an das Linienende
	rParam.aJoin1.X() = rEnd.X() + nDxW / 2;
	rParam.aJoin1.Y() = rEnd.Y() + nDyW / 2;
	rParam.aJoin2 = rParam.aJoin1;
	rParam.aJoin2.X() -= nDxW;
	rParam.aJoin2.Y() -= nDyW;
	rParam.bHasJoin = FALSE;
	rParam.bUseJoin3 = FALSE;

	// Anschlusslinie vorhanden?
	if ( pNext )
		CalcFatLineJoin(rEnd, *pNext, rParam);

	while ( !bLineComplete )
	{
		double fSegLength;
		// nPattern < 0: durchgehende Linie
		if ( nPattern < 0 ) fSegLength = 1.0;
		else                fSegLength = (double) nPattern / fLength;
		fDx += fSegLength * nLineDx;
		fDy += fSegLength * nLineDy;
		nDx = (long) fDx;
		nDy = (long) fDy;
		fDx -= nDx;     // Rundungsfehler ausgleichen
		fDy -= nDy;
		aPoly[2].X() += nDx;
		aPoly[2].Y() += nDy;
		aPoly[3].X() += nDx;
		aPoly[3].Y() += nDy;
		SegEnd.X() += nDx;
		SegEnd.Y() += nDy;

		// wenn das rEnde ueberschritten wurde, hat das Vorzeichen
		// der Abstaende vom rEndpunkt gewechselt; durch Xor-Verknuepfung
		// wird dieser Wechsel festgestellt
		long nEndDiffX = (SegEnd.X() - rEnd.X());
		long nEndDiffY = (SegEnd.Y() - rEnd.Y());

		if ( (nEndDiffX ^ (SegStart.X() - rEnd.X())) < 0 ||
			 (nEndDiffY ^ (SegStart.Y() - rEnd.Y())) < 0 ||
			 (!nEndDiffX && !nEndDiffY) )
		{
			if ( nDx || nDy )
			{
				if ( Abs(nDx) >= Abs(nDy) )
				{
					long nDiffX = SegEnd.X() - rEnd.X();
					rParam.nPatRemain = nPattern * nDiffX / nDx;
				}
				else
				{
					long nDiffY = SegEnd.Y() - rEnd.Y();
					rParam.nPatRemain = nPattern * nDiffY / nDy;
				}
			}
			else
				rParam.nPatRemain = 0;

			rParam.nPatSeg = nSeg;
			if ( rParam.bUseJoin3 )
			{
				aPoly[2] = rParam.aJoin1;
				aPoly[3] = rParam.aJoin3;
			}
			else
			{
				aPoly[2] = rParam.aJoin2;
				aPoly[3] = rParam.aJoin1;
			}
			bLineComplete = TRUE;
		}
		if ( !(nSeg & 0x1) )
			pOut->DrawPolygon(aPoly);

		aPoly[0] = aPoly[3];
		aPoly[1] = aPoly[2];
		aPoly[4] = aPoly[0];
		SegStart = SegEnd;

		if ( pLinePattern )
		{
			nSeg++;
			if ( pLinePattern[nSeg] == 0 )
				nSeg = 0;
			nPattern = pLinePattern[nSeg];
		}
	}
}

/*************************************************************************
|*
|*    XOutputDevice::DrawPatternLine()
|*
|*    Beschreibung      Haarlinie mit Linienstil zeichnen
|*    Ersterstellung    13.08.95 ESO
|*    Letzte Aenderung  14.08.95 ESO
|*
\************************************************************************/

void XOutputDevice::DrawPatternLine(const Point& rStart, const Point& rEnd,
									XLineParam& rParam)
{
	Point   aP1, aP2;
	BOOL    bLineComplete = FALSE;

	long    nLineDx = rParam.nLineDx;
	long    nLineDy = rParam.nLineDy;
	double  fLength = rParam.fLength;

	double  fDx = 0, fDy = 0;
	long    nDx, nDy;
	long    nSeg = rParam.nPatSeg;
	long    nPattern;

	// Linienmuster vorhanden?
	if ( !pLinePattern )
	{
		nPattern = -1;
		nSeg = 0;
	}
	else
	{
		// Angefangenes Segment zu Ende zeichnen?
		if ( rParam.nPatRemain )
			nPattern = rParam.nPatRemain;
		else    // sonst naechstes Segment
		{
			nSeg++;
			if ( pLinePattern[nSeg] == 0 )  nSeg = 0;
			nPattern = pLinePattern[nSeg];
		}
	}
	// Punkte fuer DrawLine initialisieren
	aP1 = rStart;
	aP2 = aP1;

	// Punkte zum Testen auf erreichen des Linienendes
	Point SegStart = rStart;
	Point SegEnd   = rStart;

	while ( !bLineComplete )
	{
		double fSegLength;
		// nPattern < 0: durchgehende Linie
		if ( nPattern < 0 ) fSegLength = 1.0;
		else                fSegLength = (double) nPattern / fLength;
		fDx += fSegLength * nLineDx;
		fDy += fSegLength * nLineDy;
		nDx = (long) fDx;
		nDy = (long) fDy;
		fDx -= nDx;     // Rundungsfehler ausgleichen
		fDy -= nDy;
		aP2.X() += nDx;
		aP2.Y() += nDy;
		SegEnd.X() += nDx;
		SegEnd.Y() += nDy;

		// wenn das Ende ueberschritten wurde, hat das Vorzeichen
		// der Abstaende vom Endpunkt gewechselt; durch Xor-Verknuepfung
		// wird dieser Wechsel festgestellt
		long nEndDiffX = (SegEnd.X() - rEnd.X());
		long nEndDiffY = (SegEnd.Y() - rEnd.Y());

		if ( (nEndDiffX ^ (SegStart.X() - rEnd.X())) < 0 ||
			 (nEndDiffY ^ (SegStart.Y() - rEnd.Y())) < 0 ||
			 (!nEndDiffX && !nEndDiffY) )
		{
			if ( nDx || nDy )
			{
				if ( Abs(nDx) >= Abs(nDy) )
				{
					long nDiffX = SegEnd.X() - rEnd.X();
					rParam.nPatRemain = nPattern * nDiffX / nDx;
				}
				else
				{
					long nDiffY = SegEnd.Y() - rEnd.Y();
					rParam.nPatRemain = nPattern * nDiffY / nDy;
				}
			}
			else
				rParam.nPatRemain = 0;

			rParam.nPatSeg = nSeg;
			aP2 = rEnd;
			bLineComplete = TRUE;
		}
		if ( !(nSeg & 0x1) )
		{
			pOut->DrawLine(aP1, aP2);
		}

		aP1 = aP2;
		SegStart = SegEnd;

		if ( pLinePattern )
		{
			nSeg++;
			if ( pLinePattern[nSeg] == 0 )
				nSeg = 0;
			nPattern = pLinePattern[nSeg];
		}
	}
}

/*************************************************************************
|*
|*    XOutputDevice::DrawLinePolygon()
|*
|*    Beschreibung      Polygon-Linie (ggf. mit Linienmuster) zeichnen
|*    Ersterstellung    28.11.94
|*    Letzte Aenderung  09.10.95 ESO
|*
\************************************************************************/

void XOutputDevice::DrawLinePolygon( const Polygon& rPoly, BOOL bClosePoly )
{
	if( nLineTransparence )
	{
		GDIMetaFile		aMtf;
		VirtualDevice	aVDev;
		OutputDevice*	pOldOut = pOut;
		MapMode			aMap( pOldOut->GetMapMode() );
		const BYTE		cTrans = sal::static_int_cast< BYTE >(
            nLineTransparence * 255 / 100 );
		const Color		aTrans( cTrans, cTrans, cTrans );
		Gradient		aTransGradient( GRADIENT_LINEAR, aTrans, aTrans );

		pOut = &aVDev;
		aVDev.EnableOutput( FALSE );
		aVDev.SetMapMode( pOldOut->GetMapMode() );
		aMtf.Record( &aVDev );
		aVDev.SetLineColor( pOldOut->GetLineColor() );
		aVDev.SetFillColor( pOldOut->GetFillColor() );
		aVDev.SetFont( pOldOut->GetFont() );
		aVDev.SetDrawMode( pOldOut->GetDrawMode() );
		aVDev.SetSettings( pOldOut->GetSettings() );
		aVDev.SetRefPoint( pOldOut->GetRefPoint() );
		ImpDrawLinePolygon( rPoly, bClosePoly );
		aMtf.Stop();
		pOut = pOldOut;

		Rectangle aBound;

		for( MetaAction* pAct = aMtf.FirstAction(); pAct; pAct = aMtf.NextAction() )
		{
			if( pAct->GetType() == META_POLYGON_ACTION )
				aBound.Union( ( (MetaPolygonAction*) pAct )->GetPolygon().GetBoundRect() );
			else if( pAct->GetType() == META_POLYLINE_ACTION )
				aBound.Union( ( (MetaPolyLineAction*) pAct )->GetPolygon().GetBoundRect() );
			else if( pAct->GetType() == META_LINE_ACTION )
			{
				const Point aStart( ( (MetaLineAction*) pAct )->GetStartPoint() );
				const Point aEnd( ( (MetaLineAction*) pAct )->GetEndPoint() );
				aBound.Union( Rectangle( aStart, aEnd ) );
			}
		}

		if( aMtf.GetActionCount() )
		{
			Size		aSizeLog( aBound.GetSize() );
			const Size	aMinSizeLog( pOut->PixelToLogic( Size( 1, 1 ) ) );
			const Size	aSizePix( pOut->LogicToPixel( aSizeLog ) );

			// watch for minimum width
			if( !aSizePix.Width() )
				aSizeLog.Width() = aMinSizeLog.Width();

			// watch for minimum width
			if( !aSizePix.Height() )
				aSizeLog.Height() = aMinSizeLog.Height();

			aMap.SetOrigin( aBound.TopLeft() );
			aMtf.SetPrefMapMode( aMap );
			aMtf.SetPrefSize( aBound.GetSize() );
			aTransGradient.SetSteps(3);
			pOut->DrawTransparent( aMtf, aBound.TopLeft(), aSizeLog, aTransGradient );
		}
	}
	else
		ImpDrawLinePolygon( rPoly, bClosePoly );
}

// -----------------------------------------------------------------------------

void XOutputDevice::ImpDrawLinePolygon(const Polygon& rPoly, BOOL bClosePoly)
{
	Polygon         aPoly( rPoly );
	const Point*    pNextPoint;
	Point           aLineStartPos, aLineEndPos;
	XLineParam      aLParam, aStartParam, aEndParam;
	USHORT          nPntMax = aPoly.GetSize() - 1;

	if( nPntMax >= 1 )
	{
		if( bHair || ( ( XLINE_SOLID == eLineStyle ) && ( nLineWidth ==  0 ) ) )
		{
			// #107240#
			// Since this method is also used for XOR drawing it's not allowed
			// to optimize to line drawing here. DrawLine() does not draw the
			// last point, thus a cycle with XOR drawing with DrawPolyLine() and then
			// deleting the last part with DrawLine() results in one point not being deleted.
			// if( 1 == nPntMax )
			//	pOut->DrawLine( rPoly[ 0 ], rPoly[ 1 ] );
			// else
			pOut->DrawPolyLine( rPoly );

			DrawLineStartEnd( rPoly );
		}
		else if( XLINE_NONE != eLineStyle )
		{
			Color		aOldLineColor;
			Color		aOldFillColor;
			Point		aDiff;
			const ULONG	nOldDrawMode = pOut->GetDrawMode();
			USHORT		i = 0;

			if( !nLineWidth )
			{
				aOldLineColor = pOut->GetLineColor();
				pOut->SetLineColor( aLineColor );
			}

			aOldFillColor = pOut->GetFillColor();

			if( nOldDrawMode & DRAWMODE_WHITEFILL )
			{
				ULONG nNewDrawMode = nOldDrawMode;

				nNewDrawMode &= ~DRAWMODE_WHITEFILL;
				nNewDrawMode |= DRAWMODE_BLACKFILL;
				pOut->SetDrawMode( nNewDrawMode );
			}

            if( nOldDrawMode & DRAWMODE_BLACKLINE )
            {
                const Color aBlack( COL_BLACK );

                pOut->SetDrawMode( pOut->GetDrawMode() & (~DRAWMODE_SETTINGSFILL) );
                pOut->SetFillColor( aBlack );
            }
            else if( nOldDrawMode & DRAWMODE_SETTINGSLINE )
            {
                pOut->SetDrawMode( pOut->GetDrawMode() & (~DRAWMODE_SETTINGSFILL) );
				svtools::ColorConfig aColorConfig;
				Color aColor( aColorConfig.GetColorValue( svtools::FONTCOLOR ).nColor );
                pOut->SetFillColor( aColor );
            }
            else
                pOut->SetFillColor( aLineColor );

			// bei einfachen Linien darf das Polygon nicht geschlossen sein (#24000)
			if ( aPoly[ nPntMax ] == aPoly[ 0 ] )
			{
				if ( nPntMax > 2 )
				{
					nPntMax--;
					bClosePoly = TRUE;
				}
				else if ( 2 == nPntMax )
					bClosePoly = FALSE;
			}

			// Linien mit Laenge 0 nicht beruecksichtigen
			while ( i < nPntMax )
			{
				aDiff = aPoly[i+1] - aPoly[0];

				if ( bLineStart && !bClosePoly )
				{
					long nSqLen = aDiff.X() * aDiff.X() + aDiff.Y() * aDiff.Y();
					if ( nSqLen > nLineStartSqLen || i == nPntMax-1 )
					{
						aLineStartPos = aPoly[0];
						aStartParam.Init(aPoly[0], aPoly[i+1], 1);
						double fLen = sqrt((double)nLineStartSqLen);
						if ( aStartParam.fLength )
							fLen /= aStartParam.fLength;
						aPoly[i].X() = aPoly[0].X() + (long) (fLen * aStartParam.nLineDx);
						aPoly[i].Y() = aPoly[0].Y() + (long) (fLen * aStartParam.nLineDy);
						break;
					}
				}
				else if ( aDiff.X() || aDiff.Y() )
					break;
				i++;
			}
			USHORT  nLastPnt = nPntMax;

			while ( nPntMax > i )
			{
				aDiff = aPoly[nPntMax-1] - aPoly[nLastPnt];
				if ( bLineEnd && !bClosePoly )
				{
					long nSqLen = aDiff.X() * aDiff.X() + aDiff.Y() * aDiff.Y();
					if ( nSqLen > nLineEndSqLen || nPntMax == i+1 )
					{
						aLineEndPos = aPoly[nLastPnt];
						aEndParam.Init(aPoly[nLastPnt], aPoly[nPntMax-1], 1);
						double fLen = sqrt((double)nLineEndSqLen);
						if ( aEndParam.fLength )
							fLen /= aEndParam.fLength;
						aPoly[nPntMax].X() = aPoly[nLastPnt].X() + (long) (fLen * aEndParam.nLineDx);
						aPoly[nPntMax].Y() = aPoly[nLastPnt].Y() + (long) (fLen * aEndParam.nLineDy);
						break;
					}
				}
				else if ( aDiff.X() || aDiff.Y() )
					break;
				nPntMax--;
			}

			if ( bClosePoly )
			{
				aDiff = aPoly[nPntMax] - aPoly[i];
				if ( !aDiff.X() && !aDiff.Y() )
					nPntMax--;
				aLParam.Init(aPoly[nPntMax], aPoly[i], nLineWidth);
				if ( nLineWidth > 0 )
					CalcFatLineJoin(aPoly[i], aPoly[i+1], aLParam);
			}
			else
				aLParam.Init(aPoly[i], aPoly[i+1], nLineWidth);

			while ( i < nPntMax )
			{
				USHORT nPos = i + 1;
				while ( nPos < nPntMax )
				{
					aDiff = aPoly[nPos+1] - aPoly[nPos];
					if ( aDiff.X() || aDiff.Y() )
						break;
					nPos++;
				}
				if ( nPos+1 <= nPntMax )    pNextPoint = &aPoly[nPos+1];
				else if ( bClosePoly )      pNextPoint = &aPoly[0];
				else                        pNextPoint = NULL;

				if ( nLineWidth > 0 )
					DrawFatLine(aPoly[i], aPoly[i+1], pNextPoint, aLParam);
				else
				{
					aLParam.nLineDx = aPoly[i+1].X() - aPoly[i].X();
					aLParam.nLineDy = aPoly[i+1].Y() - aPoly[i].Y();
					aLParam.fLength = sqrt((double) aLParam.nLineDx * aLParam.nLineDx +
										   (double) aLParam.nLineDy * aLParam.nLineDy);
					DrawPatternLine(aPoly[i], aPoly[i+1], aLParam);
				}
				i = nPos;
			}
			if ( bClosePoly )
			{
				if ( nLineWidth > 0 )
					DrawFatLine(aPoly[i], aPoly[0], &aPoly[1], aLParam);
				else
				{
					aLParam.nLineDx = aPoly[0].X() - aPoly[i].X();
					aLParam.nLineDy = aPoly[0].Y() - aPoly[i].Y();
					aLParam.fLength = sqrt((double) aLParam.nLineDx * aLParam.nLineDx +
										   (double) aLParam.nLineDy * aLParam.nLineDy);
					DrawPatternLine(aPoly[i], aPoly[0], aLParam);
				}
			}
			else
			{
				if ( bLineStart )
					DrawStartEndPoly(aLineStartPos, maLineStartPolyPolygon, aStartParam);
				if ( bLineEnd )
					DrawStartEndPoly(aLineEndPos, maLineEndPolyPolygon, aEndParam);
			}

			if( nLineWidth == 0 )
				pOut->SetLineColor( aOldLineColor );

			pOut->SetFillColor( aOldFillColor );
			pOut->SetDrawMode( nOldDrawMode );
		}
	}
}

// eof
